@@ -12,20 +12,87 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from __future__ import annotations
 
+
+import functools
 import glob
 import os
+import typing
+from typing import Optional
 
-from oslo_concurrency import processutils as putils
+from oslo_concurrency import lockutils
+from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import reflection
+from oslo_utils import timeutils
 
 from os_brick import exception
 from os_brick import initiator
 from os_brick.initiator import host_driver
 from os_brick.initiator import initiator_connector
 from os_brick.initiator import linuxscsi
+from os_brick import utils
 
 LOG = logging.getLogger(__name__)
+CONF = cfg.CONF
+
+
+def synchronized(name, lock_file_prefix='os-brick-', external=False,
+                 lock_path=None, semaphores=None, delay=0.01, fair=False,
+                 blocking=True):
+    """os-brick synchronization decorator
+
+    Like the one in lock_utils but defaulting the prefix to os-brick- and using
+    our own lock_path.
+
+    Cannot use lock_utils one because when using the default we don't know the
+    value until setup has been called, which can be after the code using the
+    decorator has been loaded.
+    """
+    def wrap(f):
+        @functools.wraps(f)
+        def inner(*args, **kwargs):
+            t1 = timeutils.now()
+            t2 = None
+            gotten = True
+            lpath = lock_path or CONF.os_brick.lock_path
+            # TODO: (AA Release) Remove this failsafe
+            if not lpath and CONF.oslo_concurrency.lock_path:
+                LOG.warning("Service needs to call os_brick.setup() before "
+                            "connecting volumes, if it doesn't it will break "
+                            "on the next release")
+                lpath = CONF.oslo_concurrency.lock_path
+            f_name = reflection.get_callable_name(f)
+            try:
+                LOG.debug('Acquiring lock "%s" by "%s"', name, f_name)
+                with lockutils.lock(name, lock_file_prefix, external, lpath,
+                                    do_log=False, semaphores=semaphores,
+                                    delay=delay, fair=fair, blocking=blocking):
+                    t2 = timeutils.now()
+                    LOG.debug('Lock "%(name)s" acquired by "%(function)s" :: '
+                              'waited %(wait_secs)0.3fs',
+                              {'name': name,
+                               'function': f_name,
+                               'wait_secs': (t2 - t1)})
+                    return f(*args, **kwargs)
+            except lockutils.AcquireLockFailedException:
+                gotten = False
+            finally:
+                t3 = timeutils.now()
+                if t2 is None:
+                    held_secs = "N/A"
+                else:
+                    held_secs = "%0.3fs" % (t3 - t2)
+                LOG.debug('Lock "%(name)s" "%(gotten)s" by "%(function)s" ::'
+                          ' held %(held_secs)s',
+                          {'name': name,
+                           'gotten': 'released' if gotten else 'unacquired',
+                           'function': f_name,
+                           'held_secs': held_secs})
+        return inner
+
+    return wrap
 
 
 class BaseLinuxConnector(initiator_connector.InitiatorConnector):
@@ -57,23 +124,11 @@ class BaseLinuxConnector(initiator_connector.InitiatorConnector):
         return props
 
     def check_valid_device(self, path: str, run_as_root: bool = True) -> bool:
-        cmd = ('dd', 'if=%(path)s' % {"path": path},
-               'of=/dev/null', 'count=1')
-        out, info = None, None
-        try:
-            out, info = self._execute(*cmd, run_as_root=run_as_root,
-                                      root_helper=self._root_helper)
-        except putils.ProcessExecutionError as e:
-            LOG.error("Failed to access the device on the path "
-                      "%(path)s: %(error)s.",
-                      {"path": path, "error": e.stderr})
-            return False
-        # If the info is none, the path does not exist.
-        if info is None:
-            return False
-        return True
+        return utils.check_valid_device(self, path)
 
-    def get_all_available_volumes(self, connection_properties=None):
+    def get_all_available_volumes(
+            self,
+            connection_properties: Optional[dict] = None) -> list:
         volumes = []
         path = self.get_search_path()
         if path:
@@ -88,7 +143,7 @@ class BaseLinuxConnector(initiator_connector.InitiatorConnector):
     def _discover_mpath_device(self,
                                device_wwn: str,
                                connection_properties: dict,
-                               device_name: str) -> tuple:
+                               device_name: str) -> tuple[str, str]:
         """This method discovers a multipath device.
 
         Discover a multipath device based on a defined connection_property
@@ -126,4 +181,7 @@ class BaseLinuxConnector(initiator_connector.InitiatorConnector):
             except exception.BlockDeviceReadOnly:
                 LOG.warning('Block device %s is still read-only. '
                             'Continuing anyway.', device_path)
+
+        device_path = typing.cast(str, device_path)
+        multipath_id = typing.cast(str, multipath_id)
         return device_path, multipath_id
