@@ -16,7 +16,6 @@ import json
 import os
 import urllib
 
-from oslo_concurrency import lockutils
 from oslo_log import log as logging
 import requests
 
@@ -30,7 +29,6 @@ from os_brick import utils
 LOG = logging.getLogger(__name__)
 DEVICE_SCAN_ATTEMPTS_DEFAULT = 3
 CONNECTOR_CONF_PATH = '/opt/emc/scaleio/openstack/connector.conf'
-synchronized = lockutils.synchronized_with_prefix('os-brick-')
 
 
 def io(_type, nr):
@@ -50,7 +48,9 @@ class ScaleIOConnector(base.BaseLinuxConnector):
 
     OK_STATUS_CODE = 200
     VOLUME_NOT_MAPPED_ERROR = 84
+    VOLUME_NOT_MAPPED_ERROR_v4 = 4039
     VOLUME_ALREADY_MAPPED_ERROR = 81
+    VOLUME_ALREADY_MAPPED_ERROR_v4 = 4037
     GET_GUID_OP_CODE = io('a', 14)
     RESCAN_VOLS_OP_CODE = io('a', 10)
 
@@ -335,8 +335,10 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         self.server_ip = connection_properties['serverIP']
         self.server_port = connection_properties['serverPort']
         self.server_username = connection_properties['serverUsername']
-        self.server_password, self.server_token = self._get_password_token(
+        self.server_password, server_token = self._get_password_token(
             connection_properties)
+        if server_token:
+            self.server_token = server_token
         self.iops_limit = connection_properties['iopsLimit']
         self.bandwidth_limit = connection_properties['bandwidthLimit']
         self.verify_certificate = (
@@ -349,7 +351,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
 
     @utils.trace
     @utils.connect_volume_prepare_result
-    @lockutils.synchronized('scaleio', 'scaleio-', external=True)
+    @base.synchronized('scaleio', 'scaleio-', external=True)
     def connect_volume(self, connection_properties):
         """Connect the volume.
 
@@ -402,7 +404,8 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         if r.status_code != self.OK_STATUS_CODE:
             response = r.json()
             error_code = response['errorCode']
-            if error_code == self.VOLUME_ALREADY_MAPPED_ERROR:
+            if error_code == self.VOLUME_ALREADY_MAPPED_ERROR or \
+               error_code == self.VOLUME_ALREADY_MAPPED_ERROR_v4:
                 LOG.warning(
                     "Ignoring error mapping volume %(volume_name)s: "
                     "volume already mapped.",
@@ -463,7 +466,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         return device_info
 
     @utils.trace
-    @lockutils.synchronized('scaleio', 'scaleio-', external=True)
+    @base.synchronized('scaleio', 'scaleio-', external=True)
     @utils.connect_volume_undo_prepare_result(unlink_after=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
@@ -516,7 +519,8 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         if r.status_code != self.OK_STATUS_CODE:
             response = r.json()
             error_code = response['errorCode']
-            if error_code == self.VOLUME_NOT_MAPPED_ERROR:
+            if error_code == self.VOLUME_NOT_MAPPED_ERROR or \
+               error_code == self.VOLUME_NOT_MAPPED_ERROR_v4:
                 LOG.warning(
                     "Ignoring error unmapping volume %(volume_id)s: "
                     "volume not mapped.", {'volume_id': self.volume_name}
@@ -539,22 +543,9 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         self._rescan_vols()
         volume_paths = self.get_volume_paths(connection_properties)
         if volume_paths:
-            return self.get_device_size(volume_paths[0])
+            return utils.get_device_size(self, volume_paths[0])
 
         # if we got here, the volume is not mapped
         msg = (_("Error extending ScaleIO volume"))
         LOG.error(msg)
         raise exception.BrickException(message=msg)
-
-    def get_device_size(self, device):
-        """Get the size in bytes of a volume."""
-        (out, _err) = self._execute('blockdev', '--getsize64',
-                                    device, run_as_root=True,
-                                    root_helper=self._root_helper)
-        var = str(out.strip())
-        LOG.debug("Device %(dev)s size: %(var)s",
-                  {'dev': device, 'var': var})
-        if var.isnumeric():
-            return int(var)
-        else:
-            return None
