@@ -12,7 +12,10 @@
 
 """Generic RBD connection utilities."""
 
+from __future__ import annotations
+
 import io
+from typing import NoReturn, Optional  # noqa: H301
 
 from oslo_log import log as logging
 
@@ -33,10 +36,12 @@ LOG = logging.getLogger(__name__)
 
 class RBDClient(object):
 
-    def __init__(self, user, pool, *args, **kwargs):
+    def __init__(self, user: str, pool: str, *args, **kwargs):
 
         self.rbd_user = user
         self.rbd_pool = pool
+        self.rados: 'rados.Rados'
+        self.rbd: 'rbd.RBD'
 
         for attr in ['rbd_user', 'rbd_pool']:
             val = getattr(self, attr)
@@ -54,9 +59,8 @@ class RBDClient(object):
             raise exception.InvalidParameterValue(
                 err=_('rbd module required'))
 
-        self.rbd_conf = kwargs.get('conffile', '/etc/ceph/ceph.conf')
-        self.rbd_cluster_name = kwargs.get('rbd_cluster_name', 'ceph')
-        self.rados_connect_timeout = kwargs.get('rados_connect_timeout', -1)
+        self.rbd_conf: str = kwargs.get('conffile', '/etc/ceph/ceph.conf')
+        self.rbd_cluster_name: str = kwargs.get('rbd_cluster_name', 'ceph')
 
         self.client, self.ioctx = self.connect()
 
@@ -66,19 +70,14 @@ class RBDClient(object):
     def __exit__(self, type_, value, traceback):
         self.disconnect()
 
-    def connect(self):
-        LOG.debug("opening connection to ceph cluster (timeout=%s).",
-                  self.rados_connect_timeout)
+    def connect(self) -> tuple['rados.Rados', 'rados.Ioctx']:
+        LOG.debug("opening connection to ceph cluster")
         client = self.rados.Rados(rados_id=self.rbd_user,
                                   clustername=self.rbd_cluster_name,
                                   conffile=self.rbd_conf)
 
         try:
-            if self.rados_connect_timeout >= 0:
-                client.connect(
-                    timeout=self.rados_connect_timeout)
-            else:
-                client.connect()
+            client.connect()
             ioctx = client.open_ioctx(self.rbd_pool)
             return client, ioctx
         except self.rados.Error:
@@ -88,7 +87,7 @@ class RBDClient(object):
             client.shutdown()
             raise exception.BrickException(message=msg)
 
-    def disconnect(self):
+    def disconnect(self) -> None:
         # closing an ioctx cannot raise an exception
         self.ioctx.close()
         self.client.shutdown()
@@ -97,7 +96,11 @@ class RBDClient(object):
 class RBDVolume(object):
     """Context manager for dealing with an existing rbd volume."""
 
-    def __init__(self, client, name, snapshot=None, read_only=False):
+    def __init__(self,
+                 client: RBDClient,
+                 name: str,
+                 snapshot: Optional[str] = None,
+                 read_only: bool = False):
         if snapshot is not None:
             snapshot = utils.convert_str(snapshot)
 
@@ -118,16 +121,16 @@ class RBDVolume(object):
         self.name = name
         self.client = client
 
-    def close(self):
+    def close(self) -> None:
         try:
             self.image.close()
         finally:
             self.client.disconnect()
 
-    def __enter__(self):
+    def __enter__(self) -> 'RBDVolume':
         return self
 
-    def __exit__(self, type_, value, traceback):
+    def __exit__(self, type_, value, traceback) -> None:
         self.close()
 
     def __getattr__(self, attrib):
@@ -136,7 +139,11 @@ class RBDVolume(object):
 
 class RBDImageMetadata(object):
     """RBD image metadata to be used with RBDVolumeIOWrapper."""
-    def __init__(self, image, pool, user, conf):
+    def __init__(self,
+                 image: 'rbd.Image',
+                 pool: Optional[str],
+                 user: Optional[str],
+                 conf: Optional[str]):
         self.image = image
         self.pool = utils.convert_str(pool or '')
         self.user = utils.convert_str(user or '')
@@ -149,33 +156,33 @@ class RBDVolumeIOWrapper(io.RawIOBase):
     Calling unimplemented interfaces will raise IOError.
     """
 
-    def __init__(self, rbd_volume):
+    def __init__(self, rbd_volume: RBDImageMetadata):
         super(RBDVolumeIOWrapper, self).__init__()
         self._rbd_volume = rbd_volume
         self._offset = 0
 
-    def _inc_offset(self, length):
+    def _inc_offset(self, length: int) -> None:
         self._offset += length
 
     @property
-    def rbd_image(self):
+    def rbd_image(self) -> 'rbd.Image':
         return self._rbd_volume.image
 
     @property
-    def rbd_user(self):
+    def rbd_user(self) -> str:
         return self._rbd_volume.user
 
     @property
-    def rbd_pool(self):
+    def rbd_pool(self) -> str:
         return self._rbd_volume.pool
 
     @property
-    def rbd_conf(self):
+    def rbd_conf(self) -> str:
         return self._rbd_volume.conf
 
-    def read(self, length=None):
+    def read(self, length: Optional[int] = None) -> bytes:
         offset = self._offset
-        total = self._rbd_volume.image.size()
+        total = int(self._rbd_volume.image.size())
 
         # NOTE(dosaboy): posix files do not barf if you read beyond their
         # length (they just return nothing) but rbd images do so we need to
@@ -198,14 +205,14 @@ class RBDVolumeIOWrapper(io.RawIOBase):
         self._inc_offset(length)
         return data
 
-    def write(self, data):
+    def write(self, data) -> None:
         self._rbd_volume.image.write(data, self._offset)
         self._inc_offset(len(data))
 
-    def seekable(self):
+    def seekable(self) -> bool:
         return True
 
-    def seek(self, offset, whence=0):
+    def seek(self, offset: int, whence: int = 0):
         if whence == 0:
             new_offset = offset
         elif whence == 1:
@@ -222,16 +229,27 @@ class RBDVolumeIOWrapper(io.RawIOBase):
 
         self._offset = new_offset
 
-    def tell(self):
+    def tell(self) -> int:
         return self._offset
 
-    def flush(self):
+    def flush(self) -> None:
+        # Raise ValueError if already closed
+        super().flush()
+        # Don't fail on flush by calling it when underlying image is closed.
         try:
-            self._rbd_volume.image.flush()
+            self.rbd_image.require_not_closed()
+        except rbd.InvalidArgument:  # Image is closed
+            LOG.warning("RBDVolumeIOWrapper's underlying image %s was closed "
+                        "directly (probably by the GC) instead of through the "
+                        "wrapper", self.rbd_image.name)
+            return
+
+        try:
+            self.rbd_image.flush()
         except AttributeError:
             LOG.warning("flush() not supported in this version of librbd")
 
-    def fileno(self):
+    def fileno(self) -> NoReturn:
         """RBD does not have support for fileno() so we raise IOError.
 
         Raising IOError is recommended way to notify caller that interface is
@@ -239,5 +257,9 @@ class RBDVolumeIOWrapper(io.RawIOBase):
         """
         raise IOError(_("fileno() not supported by RBD()"))
 
-    def close(self):
-        self.rbd_image.close()
+    def close(self) -> None:
+        if not self.closed:
+            # Can't set closed attribute ourselves, call parent to flush and
+            # change it.
+            super().close()
+            self.rbd_image.close()
