@@ -12,6 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import builtins
 import functools
 import io
 import time
@@ -28,6 +29,86 @@ class WrongException(exception.BrickException):
     pass
 
 
+@ddt.ddt
+class TestUtils(base.TestCase):
+    @ddt.data(('1024', 1024), ('junk', None), ('2048\n', 2048))
+    @ddt.unpack
+    def test_get_device_size(self, cmd_out, expected):
+        mock_execute = mock.Mock()
+        mock_execute._execute.return_value = (cmd_out, None)
+
+        device = '/dev/fake'
+        ret_size = utils.get_device_size(mock_execute, device)
+        self.assertEqual(expected, ret_size)
+        mock_execute._execute.assert_called_once_with(
+            'blockdev', '--getsize64', device,
+            run_as_root=True, root_helper=mock_execute._root_helper)
+
+    @mock.patch.object(builtins, 'open')
+    def test_get_nvme_host_id_file_available(self, mock_open):
+        mock_open.return_value.__enter__.return_value.read.return_value = (
+            'uuid\n')
+        result = utils.get_nvme_host_id(mock.sentinel.uuid)
+        mock_open.assert_called_once_with('/etc/nvme/hostid', 'r')
+        self.assertEqual('uuid', result)
+
+    @mock.patch.object(utils.priv_nvme, 'create_hostid')
+    @mock.patch.object(builtins, 'open')
+    def test_get_nvme_host_id_io_err(self, mock_open, mock_create):
+        mock_create.return_value = mock.sentinel.uuid_return
+        mock_open.side_effect = IOError()
+        result = utils.get_nvme_host_id(mock.sentinel.uuid)
+        mock_open.assert_called_once_with('/etc/nvme/hostid', 'r')
+        mock_create.assert_called_once_with(mock.sentinel.uuid)
+        self.assertEqual(mock.sentinel.uuid_return, result)
+
+    @mock.patch('uuid.uuid4')
+    @mock.patch.object(utils.priv_nvme, 'create_hostid')
+    @mock.patch.object(builtins, 'open')
+    def test_get_nvme_host_id_io_err_no_uuid(self, mock_open, mock_create,
+                                             mock_uuid):
+        mock_create.return_value = mock.sentinel.uuid_return
+        mock_open.side_effect = IOError()
+        result = utils.get_nvme_host_id(None)
+        mock_open.assert_called_once_with('/etc/nvme/hostid', 'r')
+        mock_create.assert_called_once_with(str(mock_uuid.return_value))
+        self.assertEqual(mock.sentinel.uuid_return, result)
+
+    @mock.patch.object(utils.priv_nvme, 'create_hostid')
+    @mock.patch.object(builtins, 'open')
+    def test_get_nvme_host_id_err(self, mock_open, mock_create):
+        mock_open.side_effect = Exception()
+        result = utils.get_nvme_host_id(None)
+        mock_open.assert_called_once_with('/etc/nvme/hostid', 'r')
+        mock_create.assert_not_called()
+        self.assertIsNone(result)
+
+    @ddt.data(('fake_info', True), (None, False))
+    @ddt.unpack
+    def test_check_valid_device(self, fake_info, expected):
+        mock_execute = mock.Mock()
+        mock_execute._execute.return_value = ('fake_out', fake_info)
+
+        fake_path = '/dev/fake'
+        is_valid = utils.check_valid_device(mock_execute, fake_path)
+        self.assertEqual(expected, is_valid)
+        mock_execute._execute.assert_called_once_with(
+            'dd', 'if=/dev/fake', 'of=/dev/null', 'count=1',
+            run_as_root=True, root_helper=mock_execute._root_helper)
+
+    def test_check_valid_device_error(self):
+        mock_execute = mock.Mock()
+        p_exception = utils.processutils.ProcessExecutionError
+        mock_execute._execute.side_effect = p_exception
+
+        fake_path = '/dev/fake'
+        is_valid = utils.check_valid_device(mock_execute, fake_path)
+        self.assertEqual(False, is_valid)
+        mock_execute._execute.assert_called_once_with(
+            'dd', 'if=/dev/fake', 'of=/dev/null', 'count=1',
+            run_as_root=True, root_helper=mock_execute._root_helper)
+
+
 class TestRetryDecorator(base.TestCase):
 
     def test_no_retry_required(self):
@@ -596,3 +677,20 @@ class ConnectionPropertiesDecoratorsTestCase(base.TestCase):
         outer_self.assertEqual('extend_volume', res)
         mock_dev_path.assert_called_once_with(symlink_path)
         mock_unlink.assert_not_called()
+
+
+@ddt.ddt
+class AnyTestCase(base.TestCase):
+    @ddt.data('hola', 1, None, {'a': 1}, {1, 2}, False)
+    def test_equal(self, what):
+        self.assertEqual(what, utils.ANY)
+        self.assertEqual(utils.ANY, what)
+
+    @ddt.data('hola', 1, None, {'a': 1}, {1, 2}, False)
+    def test_different(self, what):
+        self.assertFalse(what != utils.ANY)  # noqa
+        self.assertFalse(utils.ANY != what)  # noqa
+        self.assertFalse(utils.ANY > what)  # noqa
+        self.assertFalse(utils.ANY < what)  # noqa
+        self.assertFalse(utils.ANY <= what)  # noqa
+        self.assertFalse(utils.ANY >= what)  # noqa
