@@ -34,7 +34,7 @@ source_suffix = '.rst'
 master_doc = 'index'
 
 # General information about the project.
-copyright = u'2015, Cinder Developers'
+copyright = '2015, Cinder Developers'
 
 # Release notes are unversioned, so we don't need to set version and release
 version = ''
@@ -48,6 +48,6 @@ html_theme = 'openstackdocs'
 
 # -- Options for openstackdocstheme -------------------------------------------
 
-repository_name = 'openstack/os-brick'
-bug_project = 'os-brick'
-bug_tag = ''
+openstackdocs_repo_name = 'openstack/os-brick'
+openstackdocs_bug_project = 'os-brick'
+openstackdocs_bug_tag = ''
